<?xml version="1.0" encoding="UTF-8"?>
<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform" xmlns:ext="http://exslt.org/common" version="1.0" extension-element-prefixes="ext">
  <xsl:output method="xml" version="1.0" encoding="UTF-8" indent="yes"/>
  <xsl:strip-space elements="*"/>
  <xsl:template match="@* | node()" mode="pass1">
    <xsl:copy>
      <xsl:apply-templates select="@* | node()" mode="pass1"/>
    </xsl:copy>
  </xsl:template>
  <xsl:template match="Measure" mode="pass1">
    <xsl:copy>
      <xsl:apply-templates select="@*" mode="pass1"/>
      <xsl:call-template name="Split">
        <xsl:with-param name="splitters" select="tick"/>
      </xsl:call-template>
    </xsl:copy>
  </xsl:template>
  <xsl:template name="Split">
    <xsl:param name="splitters"/>
    <!-- for the first run, initialise to empty node-set -->
    <xsl:param name="prefix" select="/.."/>
    <xsl:choose>
      <xsl:when test="$splitters">
        <xsl:variable name="prefixlen" select="count($prefix)"/>
        <xsl:for-each select="$splitters[1]">
          <xsl:variable name="nodes-upto" select="preceding-sibling::node()"/>
          <xsl:variable name="nodes-this" select="$nodes-upto[count(. | $prefix) != $prefixlen]"/>
          <voice>
            <xsl:copy-of select="$nodes-this"/>
          </voice>
          <xsl:if test="not(following-sibling::tick)">
            <voice>
              <xsl:copy-of select="current()"/>
              <xsl:copy-of select="following-sibling::node()"/>
            </voice>
          </xsl:if>
          <xsl:if test="count($splitters) &gt; 1">
            <xsl:call-template name="Split">
              <xsl:with-param name="splitters" select="$splitters[position() &gt; 1]"/>
              <xsl:with-param name="prefix" select="$nodes-upto"/>
            </xsl:call-template>
          </xsl:if>
        </xsl:for-each>
      </xsl:when>
      <xsl:otherwise>
        <voice>
          <xsl:apply-templates select="current()/*" mode="pass1"/>
        </voice>
      </xsl:otherwise>
    </xsl:choose>
  </xsl:template>
  <xsl:template match="/" mode="pass2">
    <lyrics>
      <xsl:for-each select="//*/Score">
        <xsl:if test="count(.//Lyrics) &gt; 0">
          <xsl:element name="part">
            <xsl:if test="name">
              <xsl:attribute name="name">
                <xsl:value-of select="name"/>
              </xsl:attribute>
            </xsl:if>
            <xsl:for-each select="Staff">
              <xsl:if test="count(.//Lyrics) &gt; 0">
                <xsl:variable select="@id" name="staffId"/>
                <xsl:element name="staff">
                  <xsl:attribute name="idx">
                    <xsl:value-of select="$staffId"/>
                  </xsl:attribute>
                  <xsl:attribute name="track">
                    <xsl:value-of select="../Part[./Staff/@id=$staffId]/trackName"/>
                  </xsl:attribute>
                  <xsl:call-template name="group-lyrics">
                    <xsl:with-param name="tree" select=".//Measure/voice[1]//Lyrics"/>
                    <xsl:with-param name="voice" select="1"/>
                  </xsl:call-template>
                  <xsl:call-template name="group-lyrics">
                    <xsl:with-param name="tree" select=".//Measure/voice[2]//Lyrics"/>
                    <xsl:with-param name="voice" select="2"/>
                  </xsl:call-template>
                  <xsl:call-template name="group-lyrics">
                    <xsl:with-param name="tree" select=".//Measure/voice[3]//Lyrics"/>
                    <xsl:with-param name="voice" select="3"/>
                  </xsl:call-template>
                  <xsl:call-template name="group-lyrics">
                    <xsl:with-param name="tree" select=".//Measure/voice[4]//Lyrics"/>
                    <xsl:with-param name="voice" select="4"/>
                  </xsl:call-template>
                </xsl:element>
              </xsl:if>
            </xsl:for-each>
          </xsl:element>
        </xsl:if>
      </xsl:for-each>
    </lyrics>
  </xsl:template>
  <xsl:template name="group-lyrics">
    <xsl:param name="tree"/>
    <xsl:param name="verse-no" select="0"/>
    <xsl:param name="voice"/>
    <xsl:if test="count($tree) &gt; 0">
      <xsl:variable name="elts" select="$tree[number(concat('0', no)) = $verse-no]"/>
      <xsl:element name="verse">
        <xsl:attribute name="voice">
          <xsl:value-of select="$voice"/>
        </xsl:attribute>
        <xsl:attribute name="nr">
          <xsl:value-of select="$verse-no + 1"/>
        </xsl:attribute>
        <xsl:for-each select="$elts">
          <xsl:if test="position() != 1 and not(syllabic='middle' or syllabic='end')">
            <xsl:text> </xsl:text>
          </xsl:if>
          <xsl:value-of select="text"/>
        </xsl:for-each>
      </xsl:element>
      <xsl:if test="count($tree) &gt; count($elts)">
        <xsl:call-template name="group-lyrics">
          <xsl:with-param name="tree" select="$tree[not(number(concat('0', no)) = $verse-no)]"/>
          <xsl:with-param name="verse-no" select="$verse-no + 1"/>
          <xsl:with-param name="voice" select="$voice"/>
        </xsl:call-template>
      </xsl:if>
    </xsl:if>
  </xsl:template>
  <xsl:template match="/museScore[starts-with(@version, '2.')]">
    <xsl:variable name="rewind">
      <xsl:copy>
        <xsl:apply-templates select="@* | node()" mode="pass1"/>
      </xsl:copy>
    </xsl:variable>
    <xsl:apply-templates select="ext:node-set($rewind)" mode="pass2"/>
  </xsl:template>
  <xsl:template match="/museScore[not(starts-with(@version, '2.'))]">
    <xsl:apply-templates select="/" mode="pass2"/>
  </xsl:template>
</xsl:stylesheet>
